@@ -14,11 +14,27 @@ import (
 
 	"github.com/hashicorp/terraform/internal/addrs"
 	"github.com/hashicorp/terraform/internal/configs/configschema"
+	"github.com/hashicorp/terraform/internal/lang/marks"
 	"github.com/hashicorp/terraform/internal/plans"
 	"github.com/hashicorp/terraform/internal/plans/objchange"
 	"github.com/hashicorp/terraform/internal/states"
 )
 
+// DiffLanguage controls the description of the resource change reasons.
+type DiffLanguage rune
+
+//go:generate go run golang.org/x/tools/cmd/stringer -type=DiffLanguage diff.go
+
+const (
+	// DiffLanguageProposedChange indicates that the change is one which is
+	// planned to be applied.
+	DiffLanguageProposedChange DiffLanguage = 'P'
+
+	// DiffLanguageDetectedDrift indicates that the change is detected drift
+	// from the configuration.
+	DiffLanguageDetectedDrift DiffLanguage = 'D'
+)
+
 // ResourceChange returns a string representation of a change to a particular
 // resource, for inclusion in user-facing plan output.
 //
@@ -32,6 +48,7 @@ func ResourceChange(
 	change *plans.ResourceInstanceChangeSrc,
 	schema *configschema.Block,
 	color *colorstring.Colorize,
+	language DiffLanguage,
 ) string {
 	addr := change.Addr
 	var buf bytes.Buffer
@@ -51,33 +68,91 @@ func ResourceChange(
 
 	switch change.Action {
 	case plans.Create:
-		buf.WriteString(color.Color(fmt.Sprintf("[bold]  # %s[reset] will be created", dispAddr)))
+		buf.WriteString(fmt.Sprintf(color.Color("[bold]  # %s[reset] will be created"), dispAddr))
 	case plans.Read:
-		buf.WriteString(color.Color(fmt.Sprintf("[bold]  # %s[reset] will be read during apply\n  # (config refers to values not yet known)", dispAddr)))
+		buf.WriteString(fmt.Sprintf(color.Color("[bold]  # %s[reset] will be read during apply\n  # (config refers to values not yet known)"), dispAddr))
 	case plans.Update:
-		buf.WriteString(color.Color(fmt.Sprintf("[bold]  # %s[reset] will be updated in-place", dispAddr)))
+		switch language {
+		case DiffLanguageProposedChange:
+			buf.WriteString(fmt.Sprintf(color.Color("[bold]  # %s[reset] will be updated in-place"), dispAddr))
+		case DiffLanguageDetectedDrift:
+			buf.WriteString(fmt.Sprintf(color.Color("[bold]  # %s[reset] has changed"), dispAddr))
+		default:
+			buf.WriteString(fmt.Sprintf(color.Color("[bold]  # %s[reset] update (unknown reason %s)"), dispAddr, language))
+		}
 	case plans.CreateThenDelete, plans.DeleteThenCreate:
 		switch change.ActionReason {
 		case plans.ResourceInstanceReplaceBecauseTainted:
-			buf.WriteString(color.Color(fmt.Sprintf("[bold]  # %s[reset] is tainted, so must be [bold][red]replaced", dispAddr)))
+			buf.WriteString(fmt.Sprintf(color.Color("[bold]  # %s[reset] is tainted, so must be [bold][red]replaced"), dispAddr))
 		case plans.ResourceInstanceReplaceByRequest:
-			buf.WriteString(color.Color(fmt.Sprintf("[bold]  # %s[reset] will be [bold][red]replaced[reset], as requested", dispAddr)))
+			buf.WriteString(fmt.Sprintf(color.Color("[bold]  # %s[reset] will be [bold][red]replaced[reset], as requested"), dispAddr))
 		default:
-			buf.WriteString(color.Color(fmt.Sprintf("[bold]  # %s[reset] must be [bold][red]replaced", dispAddr)))
+			buf.WriteString(fmt.Sprintf(color.Color("[bold]  # %s[reset] must be [bold][red]replaced"), dispAddr))
 		}
 	case plans.Delete:
-		buf.WriteString(color.Color(fmt.Sprintf("[bold]  # %s[reset] will be [bold][red]destroyed", dispAddr)))
+		switch language {
+		case DiffLanguageProposedChange:
+			buf.WriteString(fmt.Sprintf(color.Color("[bold]  # %s[reset] will be [bold][red]destroyed"), dispAddr))
+		case DiffLanguageDetectedDrift:
+			buf.WriteString(fmt.Sprintf(color.Color("[bold]  # %s[reset] has been deleted"), dispAddr))
+		default:
+			buf.WriteString(fmt.Sprintf(color.Color("[bold]  # %s[reset] delete (unknown reason %s)"), dispAddr, language))
+		}
+		// We can sometimes give some additional detail about why we're
+		// proposing to delete. We show this as additional notes, rather than
+		// as additional wording in the main action statement, in an attempt
+		// to make the "will be destroyed" message prominent and consistent
+		// in all cases, for easier scanning of this often-risky action.
+		switch change.ActionReason {
+		case plans.ResourceInstanceDeleteBecauseNoResourceConfig:
+			buf.WriteString(fmt.Sprintf("\n  # (because %s is not in configuration)", addr.Resource.Resource))
+		case plans.ResourceInstanceDeleteBecauseNoModule:
+			// FIXME: Ideally we'd truncate addr.Module to reflect the earliest
+			// step that doesn't exist, so it's clearer which call this refers
+			// to, but we don't have enough information out here in the UI layer
+			// to decide that; only the "expander" in Terraform Core knows
+			// which module instance keys are actually declared.
+			buf.WriteString(fmt.Sprintf("\n  # (because %s is not in configuration)", addr.Module))
+		case plans.ResourceInstanceDeleteBecauseWrongRepetition:
+			// We have some different variations of this one
+			switch addr.Resource.Key.(type) {
+			case nil:
+				buf.WriteString("\n  # (because resource uses count or for_each)")
+			case addrs.IntKey:
+				buf.WriteString("\n  # (because resource does not use count)")
+			case addrs.StringKey:
+				buf.WriteString("\n  # (because resource does not use for_each)")
+			}
+		case plans.ResourceInstanceDeleteBecauseCountIndex:
+			buf.WriteString(fmt.Sprintf("\n  # (because index %s is out of range for count)", addr.Resource.Key))
+		case plans.ResourceInstanceDeleteBecauseEachKey:
+			buf.WriteString(fmt.Sprintf("\n  # (because key %s is not in for_each map)", addr.Resource.Key))
+		}
 		if change.DeposedKey != states.NotDeposed {
 			// Some extra context about this unusual situation.
-			buf.WriteString(color.Color(fmt.Sprint("\n  # (left over from a partially-failed replacement of this instance)")))
+			buf.WriteString(color.Color("\n  # (left over from a partially-failed replacement of this instance)"))
 		}
+	case plans.NoOp:
+		if change.Moved() {
+			buf.WriteString(fmt.Sprintf(color.Color("[bold]  # %s[reset] has moved to [bold]%s[reset]"), change.PrevRunAddr.String(), dispAddr))
+			break
+		}
+		fallthrough
 	default:
 		// should never happen, since the above is exhaustive
 		buf.WriteString(fmt.Sprintf("%s has an action the plan renderer doesn't support (this is a bug)", dispAddr))
 	}
 	buf.WriteString(color.Color("[reset]\n"))
 
-	buf.WriteString(color.Color(DiffActionSymbol(change.Action)) + " ")
+	if change.Moved() && change.Action != plans.NoOp {
+		buf.WriteString(fmt.Sprintf(color.Color("  # [reset](moved from %s)\n"), change.PrevRunAddr.String()))
+	}
+
+	if change.Moved() && change.Action == plans.NoOp {
+		buf.WriteString("    ")
+	} else {
+		buf.WriteString(color.Color(DiffActionSymbol(change.Action)) + " ")
+	}
 
 	switch addr.Resource.Resource.Mode {
 	case addrs.ManagedResourceMode:
@@ -139,147 +214,6 @@ func ResourceChange(
 	return buf.String()
 }
 
-// ResourceInstanceDrift returns a string representation of a change to a
-// particular resource instance that was made outside of Terraform, for
-// reporting a change that has already happened rather than one that is planned.
-//
-// The the two resource instances have equal current objects then the result
-// will be an empty string to indicate that there is no drift to render.
-//
-// The resource schema must be provided along with the change so that the
-// formatted change can reflect the configuration structure for the associated
-// resource.
-//
-// If "color" is non-nil, it will be used to color the result. Otherwise,
-// no color codes will be included.
-func ResourceInstanceDrift(
-	addr addrs.AbsResourceInstance,
-	before, after *states.ResourceInstance,
-	schema *configschema.Block,
-	color *colorstring.Colorize,
-) string {
-	var buf bytes.Buffer
-
-	if color == nil {
-		color = &colorstring.Colorize{
-			Colors:  colorstring.DefaultColors,
-			Disable: true,
-			Reset:   false,
-		}
-	}
-
-	dispAddr := addr.String()
-	action := plans.Update
-
-	switch {
-	case before == nil || before.Current == nil:
-		// before should never be nil, but before.Current can be if the
-		// instance was deposed. There is nothing to render for a deposed
-		// instance, since we intend to remove it.
-		return ""
-
-	case after == nil || after.Current == nil:
-		// The object was deleted
-		buf.WriteString(color.Color(fmt.Sprintf("[bold]  # %s[reset] has been deleted", dispAddr)))
-		action = plans.Delete
-	default:
-		// The object was changed
-		buf.WriteString(color.Color(fmt.Sprintf("[bold]  # %s[reset] has been changed", dispAddr)))
-	}
-
-	buf.WriteString(color.Color("[reset]\n"))
-
-	buf.WriteString(color.Color(DiffActionSymbol(action)) + " ")
-
-	switch addr.Resource.Resource.Mode {
-	case addrs.ManagedResourceMode:
-		buf.WriteString(fmt.Sprintf(
-			"resource %q %q",
-			addr.Resource.Resource.Type,
-			addr.Resource.Resource.Name,
-		))
-	case addrs.DataResourceMode:
-		buf.WriteString(fmt.Sprintf(
-			"data %q %q ",
-			addr.Resource.Resource.Type,
-			addr.Resource.Resource.Name,
-		))
-	default:
-		// should never happen, since the above is exhaustive
-		buf.WriteString(addr.String())
-	}
-
-	buf.WriteString(" {")
-
-	p := blockBodyDiffPrinter{
-		buf:    &buf,
-		color:  color,
-		action: action,
-	}
-
-	// Most commonly-used resources have nested blocks that result in us
-	// going at least three traversals deep while we recurse here, so we'll
-	// start with that much capacity and then grow as needed for deeper
-	// structures.
-	path := make(cty.Path, 0, 3)
-
-	ty := schema.ImpliedType()
-
-	var err error
-	var oldObj, newObj *states.ResourceInstanceObject
-	oldObj, err = before.Current.Decode(ty)
-	if err != nil {
-		// We shouldn't encounter errors here because Terraform Core should've
-		// made sure that the previous run object conforms to the current
-		// schema by having the provider upgrade it, but we'll be robust here
-		// in case there are some edges we didn't find yet.
-		return fmt.Sprintf("  # %s previous run state doesn't conform to current schema; this is a Terraform bug\n  # %s\n", addr, err)
-	}
-	if after != nil && after.Current != nil {
-		newObj, err = after.Current.Decode(ty)
-		if err != nil {
-			// We shouldn't encounter errors here because Terraform Core should've
-			// made sure that the prior state object conforms to the current
-			// schema by having the provider upgrade it, even if we skipped
-			// refreshing on this run, but we'll be robust here in case there are
-			// some edges we didn't find yet.
-			return fmt.Sprintf("  # %s refreshed state doesn't conform to current schema; this is a Terraform bug\n  # %s\n", addr, err)
-		}
-	}
-
-	oldVal := oldObj.Value
-	var newVal cty.Value
-	if newObj != nil {
-		newVal = newObj.Value
-	} else {
-		newVal = cty.NullVal(ty)
-	}
-
-	if newVal.RawEquals(oldVal) {
-		// Nothing to show, then.
-		return ""
-	}
-
-	// We currently have an opt-out that permits the legacy SDK to return values
-	// that defy our usual conventions around handling of nesting blocks. To
-	// avoid the rendering code from needing to handle all of these, we'll
-	// normalize first.
-	// (Ideally we'd do this as part of the SDK opt-out implementation in core,
-	// but we've added it here for now to reduce risk of unexpected impacts
-	// on other code in core.)
-	oldVal = objchange.NormalizeObjectFromLegacySDK(oldVal, schema)
-	newVal = objchange.NormalizeObjectFromLegacySDK(newVal, schema)
-
-	result := p.writeBlockBodyDiff(schema, oldVal, newVal, 6, path)
-	if result.bodyWritten {
-		buf.WriteString("\n")
-		buf.WriteString(strings.Repeat(" ", 4))
-	}
-	buf.WriteString("}\n")
-
-	return buf.String()
-}
-
 // OutputChanges returns a string representation of a set of changes to output
 // values for inclusion in user-facing plan output.
 //
@@ -386,7 +320,7 @@ func (p *blockBodyDiffPrinter) writeBlockBodyDiff(schema *configschema.Block, ol
 			}
 			p.buf.WriteString("\n")
 			p.buf.WriteString(strings.Repeat(" ", indent+2))
-			p.buf.WriteString(p.color.Color(fmt.Sprintf("[dark_gray]# (%d unchanged %s hidden)[reset]", result.skippedBlocks, noun)))
+			p.buf.WriteString(fmt.Sprintf(p.color.Color("[dark_gray]# (%d unchanged %s hidden)[reset]"), result.skippedBlocks, noun))
 		}
 	}
 
@@ -532,10 +466,10 @@ func (p *blockBodyDiffPrinter) writeNestedAttrDiff(
 		if action != plans.NoOp && (p.pathForcesNewResource(path) || p.pathForcesNewResource(path[:len(path)-1])) {
 			p.buf.WriteString(p.color.Color(forcesNewResourceCaption))
 		}
-		p.writeAttrsDiff(objS.Attributes, old, new, indent+2, path, result)
-		p.writeSkippedAttr(result.skippedAttributes, indent+4)
+		p.writeAttrsDiff(objS.Attributes, old, new, indent+4, path, result)
+		p.writeSkippedAttr(result.skippedAttributes, indent+6)
 		p.buf.WriteString("\n")
-		p.buf.WriteString(strings.Repeat(" ", indent))
+		p.buf.WriteString(strings.Repeat(" ", indent+2))
 		p.buf.WriteString("}")
 
 	case configschema.NestingList:
@@ -544,9 +478,6 @@ func (p *blockBodyDiffPrinter) writeNestedAttrDiff(
 			p.buf.WriteString(p.color.Color(forcesNewResourceCaption))
 		}
 		p.buf.WriteString("\n")
-		p.buf.WriteString(strings.Repeat(" ", indent+4))
-		p.writeActionSymbol(action)
-		p.buf.WriteString("{")
 
 		oldItems := ctyCollectionValues(old)
 		newItems := ctyCollectionValues(new)
@@ -560,62 +491,91 @@ func (p *blockBodyDiffPrinter) writeNestedAttrDiff(
 		// commonLen is number of elements that exist in both lists, which
 		// will be presented as updates (~). Any additional items in one
 		// of the lists will be presented as either creates (+) or deletes (-)
-		// depending on which list they belong to.
+		// depending on which list they belong to. maxLen is the number of
+		// elements in that longer list.
 		var commonLen int
+		var maxLen int
 		// unchanged is the number of unchanged elements
 		var unchanged int
 
 		switch {
 		case len(oldItems) < len(newItems):
 			commonLen = len(oldItems)
+			maxLen = len(newItems)
 		default:
 			commonLen = len(newItems)
+			maxLen = len(oldItems)
 		}
-		for i := 0; i < commonLen; i++ {
+		for i := 0; i < maxLen; i++ {
 			path := append(path, cty.IndexStep{Key: cty.NumberIntVal(int64(i))})
-			oldItem := oldItems[i]
-			newItem := newItems[i]
-			if oldItem.RawEquals(newItem) {
+
+			var action plans.Action
+			var oldItem, newItem cty.Value
+			switch {
+			case i < commonLen:
+				oldItem = oldItems[i]
+				newItem = newItems[i]
+				if oldItem.RawEquals(newItem) {
+					action = plans.NoOp
+					unchanged++
+				} else {
+					action = plans.Update
+				}
+			case i < len(oldItems):
+				oldItem = oldItems[i]
+				newItem = cty.NullVal(oldItem.Type())
+				action = plans.Delete
+			case i < len(newItems):
+				newItem = newItems[i]
+				oldItem = cty.NullVal(newItem.Type())
+				action = plans.Create
+			default:
 				action = plans.NoOp
-				unchanged++
 			}
+
 			if action != plans.NoOp {
-				p.writeAttrsDiff(objS.Attributes, oldItem, newItem, indent+6, path, result)
-				p.writeSkippedAttr(result.skippedAttributes, indent+8)
+				p.buf.WriteString(strings.Repeat(" ", indent+4))
+				p.writeActionSymbol(action)
+				p.buf.WriteString("{")
+
+				result := &blockBodyDiffResult{}
+				p.writeAttrsDiff(objS.Attributes, oldItem, newItem, indent+8, path, result)
+				if action == plans.Update {
+					p.writeSkippedAttr(result.skippedAttributes, indent+10)
+				}
 				p.buf.WriteString("\n")
+
+				p.buf.WriteString(strings.Repeat(" ", indent+6))
+				p.buf.WriteString("},\n")
 			}
 		}
-		for i := commonLen; i < len(oldItems); i++ {
-			path := append(path, cty.IndexStep{Key: cty.NumberIntVal(int64(i))})
-			oldItem := oldItems[i]
-			newItem := cty.NullVal(oldItem.Type())
-			p.writeAttrsDiff(objS.Attributes, oldItem, newItem, indent+6, path, result)
-			p.buf.WriteString("\n")
-		}
-		for i := commonLen; i < len(newItems); i++ {
-			path := append(path, cty.IndexStep{Key: cty.NumberIntVal(int64(i))})
-			newItem := newItems[i]
-			oldItem := cty.NullVal(newItem.Type())
-			p.writeAttrsDiff(objS.Attributes, oldItem, newItem, indent+6, path, result)
-			p.buf.WriteString("\n")
-		}
-		p.buf.WriteString(strings.Repeat(" ", indent+4))
-		p.buf.WriteString("},\n")
-		p.writeSkippedElems(unchanged, indent+4)
+		p.writeSkippedElems(unchanged, indent+6)
 		p.buf.WriteString(strings.Repeat(" ", indent+2))
 		p.buf.WriteString("]")
 
+		if !new.IsKnown() {
+			p.buf.WriteString(" -> (known after apply)")
+		}
+
 	case configschema.NestingSet:
 		oldItems := ctyCollectionValues(old)
 		newItems := ctyCollectionValues(new)
 
-		allItems := make([]cty.Value, 0, len(oldItems)+len(newItems))
-		allItems = append(allItems, oldItems...)
-		allItems = append(allItems, newItems...)
-		all := cty.SetVal(allItems)
+		var all cty.Value
+		if len(oldItems)+len(newItems) > 0 {
+			allItems := make([]cty.Value, 0, len(oldItems)+len(newItems))
+			allItems = append(allItems, oldItems...)
+			allItems = append(allItems, newItems...)
+
+			all = cty.SetVal(allItems)
+		} else {
+			all = cty.SetValEmpty(old.Type().ElementType())
+		}
 
 		p.buf.WriteString(" = [")
 
+		var unchanged int
+
 		for it := all.ElementIterator(); it.Next(); {
 			_, val := it.Element()
 			var action plans.Action
@@ -624,11 +584,18 @@ func (p *blockBodyDiffPrinter) writeNestedAttrDiff(
 			case !val.IsKnown():
 				action = plans.Update
 				newValue = val
-			case !old.HasElement(val).True():
+			case !new.IsKnown():
+				action = plans.Delete
+				// the value must have come from the old set
+				oldValue = val
+				// Mark the new val as null, but the entire set will be
+				// displayed as "(unknown after apply)"
+				newValue = cty.NullVal(val.Type())
+			case old.IsNull() || !old.HasElement(val).True():
 				action = plans.Create
 				oldValue = cty.NullVal(val.Type())
 				newValue = val
-			case !new.HasElement(val).True():
+			case new.IsNull() || !new.HasElement(val).True():
 				action = plans.Delete
 				oldValue = val
 				newValue = cty.NullVal(val.Type())
@@ -638,26 +605,36 @@ func (p *blockBodyDiffPrinter) writeNestedAttrDiff(
 				newValue = val
 			}
 
+			if action == plans.NoOp {
+				unchanged++
+				continue
+			}
+
 			p.buf.WriteString("\n")
 			p.buf.WriteString(strings.Repeat(" ", indent+4))
 			p.writeActionSymbol(action)
 			p.buf.WriteString("{")
 
-			if action != plans.NoOp && (p.pathForcesNewResource(path) || p.pathForcesNewResource(path[:len(path)-1])) {
+			if p.pathForcesNewResource(path) || p.pathForcesNewResource(path[:len(path)-1]) {
 				p.buf.WriteString(p.color.Color(forcesNewResourceCaption))
 			}
 
 			path := append(path, cty.IndexStep{Key: val})
-			p.writeAttrsDiff(objS.Attributes, oldValue, newValue, indent+6, path, result)
+			p.writeAttrsDiff(objS.Attributes, oldValue, newValue, indent+8, path, result)
 
 			p.buf.WriteString("\n")
-			p.buf.WriteString(strings.Repeat(" ", indent+4))
+			p.buf.WriteString(strings.Repeat(" ", indent+6))
 			p.buf.WriteString("},")
 		}
 		p.buf.WriteString("\n")
+		p.writeSkippedElems(unchanged, indent+6)
 		p.buf.WriteString(strings.Repeat(" ", indent+2))
 		p.buf.WriteString("]")
 
+		if !new.IsKnown() {
+			p.buf.WriteString(" -> (known after apply)")
+		}
+
 	case configschema.NestingMap:
 		// For the sake of handling nested blocks, we'll treat a null map
 		// the same as an empty map since the config language doesn't
@@ -666,7 +643,12 @@ func (p *blockBodyDiffPrinter) writeNestedAttrDiff(
 		new = ctyNullBlockMapAsEmpty(new)
 
 		oldItems := old.AsValueMap()
-		newItems := new.AsValueMap()
+
+		newItems := map[string]cty.Value{}
+
+		if new.IsKnown() {
+			newItems = new.AsValueMap()
+		}
 
 		allKeys := make(map[string]bool)
 		for k := range oldItems {
@@ -688,6 +670,7 @@ func (p *blockBodyDiffPrinter) writeNestedAttrDiff(
 		for _, k := range allKeysOrder {
 			var action plans.Action
 			oldValue := oldItems[k]
+
 			newValue := newItems[k]
 			switch {
 			case oldValue == cty.NilVal:
@@ -712,20 +695,21 @@ func (p *blockBodyDiffPrinter) writeNestedAttrDiff(
 				}
 
 				path := append(path, cty.IndexStep{Key: cty.StringVal(k)})
-				p.writeAttrsDiff(objS.Attributes, oldValue, newValue, indent+6, path, result)
-				p.writeSkippedAttr(result.skippedAttributes, indent+8)
+				p.writeAttrsDiff(objS.Attributes, oldValue, newValue, indent+8, path, result)
+				p.writeSkippedAttr(result.skippedAttributes, indent+10)
 				p.buf.WriteString("\n")
-				p.buf.WriteString(strings.Repeat(" ", indent+4))
+				p.buf.WriteString(strings.Repeat(" ", indent+6))
 				p.buf.WriteString("},\n")
 			}
 		}
 
-		p.writeSkippedElems(unchanged, indent+4)
+		p.writeSkippedElems(unchanged, indent+6)
 		p.buf.WriteString(strings.Repeat(" ", indent+2))
 		p.buf.WriteString("}")
+		if !new.IsKnown() {
+			p.buf.WriteString(" -> (known after apply)")
+		}
 	}
-
-	return
 }
 
 func (p *blockBodyDiffPrinter) writeNestedBlockDiffs(name string, blockS *configschema.NestedBlock, old, new cty.Value, blankBefore bool, indent int, path cty.Path) int {
@@ -739,7 +723,7 @@ func (p *blockBodyDiffPrinter) writeNestedBlockDiffs(name string, blockS *config
 	// If either the old or the new value is marked,
 	// Display a special diff because it is irrelevant
 	// to list all obfuscated attributes as (sensitive)
-	if old.IsMarked() || new.IsMarked() {
+	if old.HasMark(marks.Sensitive) || new.HasMark(marks.Sensitive) {
 		p.writeSensitiveNestedBlockDiff(name, old, new, indent, blankBefore, path)
 		return 0
 	}
@@ -1018,7 +1002,7 @@ func (p *blockBodyDiffPrinter) writeNestedBlockDiff(name string, label *string,
 
 func (p *blockBodyDiffPrinter) writeValue(val cty.Value, action plans.Action, indent int) {
 	// Could check specifically for the sensitivity marker
-	if val.IsMarked() {
+	if val.HasMark(marks.Sensitive) {
 		p.buf.WriteString("(sensitive)")
 		return
 	}
@@ -1183,7 +1167,7 @@ func (p *blockBodyDiffPrinter) writeValueDiff(old, new cty.Value, indent int, pa
 	// However, these specialized implementations can apply only if both
 	// values are known and non-null.
 	if old.IsKnown() && new.IsKnown() && !old.IsNull() && !new.IsNull() && typesEqual {
-		if old.IsMarked() || new.IsMarked() {
+		if old.HasMark(marks.Sensitive) || new.HasMark(marks.Sensitive) {
 			p.buf.WriteString("(sensitive)")
 			if p.pathForcesNewResource(path) {
 				p.buf.WriteString(p.color.Color(forcesNewResourceCaption))
@@ -1378,7 +1362,7 @@ func (p *blockBodyDiffPrinter) writeValueDiff(old, new cty.Value, indent int, pa
 				if suppressedElements == 1 {
 					noun = "element"
 				}
-				p.buf.WriteString(p.color.Color(fmt.Sprintf("[dark_gray]# (%d unchanged %s hidden)[reset]", suppressedElements, noun)))
+				p.buf.WriteString(fmt.Sprintf(p.color.Color("[dark_gray]# (%d unchanged %s hidden)[reset]"), suppressedElements, noun))
 				p.buf.WriteString("\n")
 			}
 
@@ -1439,7 +1423,7 @@ func (p *blockBodyDiffPrinter) writeValueDiff(old, new cty.Value, indent int, pa
 						if hidden == 1 {
 							noun = "element"
 						}
-						p.buf.WriteString(p.color.Color(fmt.Sprintf("[dark_gray]# (%d unchanged %s hidden)[reset]", hidden, noun)))
+						p.buf.WriteString(fmt.Sprintf(p.color.Color("[dark_gray]# (%d unchanged %s hidden)[reset]"), hidden, noun))
 						p.buf.WriteString("\n")
 					}
 
@@ -1554,7 +1538,7 @@ func (p *blockBodyDiffPrinter) writeValueDiff(old, new cty.Value, indent int, pa
 				switch action {
 				case plans.Create, plans.NoOp:
 					v := new.Index(kV)
-					if v.IsMarked() {
+					if v.HasMark(marks.Sensitive) {
 						p.buf.WriteString("(sensitive)")
 					} else {
 						p.writeValue(v, action, indent+4)
@@ -1564,7 +1548,7 @@ func (p *blockBodyDiffPrinter) writeValueDiff(old, new cty.Value, indent int, pa
 					newV := cty.NullVal(oldV.Type())
 					p.writeValueDiff(oldV, newV, indent+4, path)
 				default:
-					if oldV.IsMarked() || newV.IsMarked() {
+					if oldV.HasMark(marks.Sensitive) || newV.HasMark(marks.Sensitive) {
 						p.buf.WriteString("(sensitive)")
 					} else {
 						p.writeValueDiff(oldV, newV, indent+4, path)
@@ -1581,7 +1565,7 @@ func (p *blockBodyDiffPrinter) writeValueDiff(old, new cty.Value, indent int, pa
 				if suppressedElements == 1 {
 					noun = "element"
 				}
-				p.buf.WriteString(p.color.Color(fmt.Sprintf("[dark_gray]# (%d unchanged %s hidden)[reset]", suppressedElements, noun)))
+				p.buf.WriteString(fmt.Sprintf(p.color.Color("[dark_gray]# (%d unchanged %s hidden)[reset]"), suppressedElements, noun))
 				p.buf.WriteString("\n")
 			}
 
@@ -1673,7 +1657,7 @@ func (p *blockBodyDiffPrinter) writeValueDiff(old, new cty.Value, indent int, pa
 				if suppressedElements == 1 {
 					noun = "element"
 				}
-				p.buf.WriteString(p.color.Color(fmt.Sprintf("[dark_gray]# (%d unchanged %s hidden)[reset]", suppressedElements, noun)))
+				p.buf.WriteString(fmt.Sprintf(p.color.Color("[dark_gray]# (%d unchanged %s hidden)[reset]"), suppressedElements, noun))
 				p.buf.WriteString("\n")
 			}
 
@@ -1744,17 +1728,17 @@ func (p *blockBodyDiffPrinter) writeSensitivityWarning(old, new cty.Value, inden
 		}
 	}
 
-	if new.IsMarked() && !old.IsMarked() {
+	if new.HasMark(marks.Sensitive) && !old.HasMark(marks.Sensitive) {
 		p.buf.WriteString(strings.Repeat(" ", indent))
-		p.buf.WriteString(p.color.Color(fmt.Sprintf("# [yellow]Warning:[reset] this %s will be marked as sensitive and will not\n", diffType)))
+		p.buf.WriteString(fmt.Sprintf(p.color.Color("# [yellow]Warning:[reset] this %s will be marked as sensitive and will not\n"), diffType))
 		p.buf.WriteString(strings.Repeat(" ", indent))
 		p.buf.WriteString(fmt.Sprintf("# display in UI output after applying this change.%s\n", valueUnchangedSuffix))
 	}
 
 	// Note if changing this attribute will change its sensitivity
-	if old.IsMarked() && !new.IsMarked() {
+	if old.HasMark(marks.Sensitive) && !new.HasMark(marks.Sensitive) {
 		p.buf.WriteString(strings.Repeat(" ", indent))
-		p.buf.WriteString(p.color.Color(fmt.Sprintf("# [yellow]Warning:[reset] this %s will no longer be marked as sensitive\n", diffType)))
+		p.buf.WriteString(fmt.Sprintf(p.color.Color("# [yellow]Warning:[reset] this %s will no longer be marked as sensitive\n"), diffType))
 		p.buf.WriteString(strings.Repeat(" ", indent))
 		p.buf.WriteString(fmt.Sprintf("# after applying this change.%s\n", valueUnchangedSuffix))
 	}
@@ -1819,7 +1803,27 @@ func ctySequenceDiff(old, new []cty.Value) []*plans.Change {
 	lcs := objchange.LongestCommonSubsequence(old, new)
 	var oldI, newI, lcsI int
 	for oldI < len(old) || newI < len(new) || lcsI < len(lcs) {
+		// We first process items in the old and new sequences which are not
+		// equal to the current common sequence item.  Old items are marked as
+		// deletions, and new items are marked as additions.
+		//
+		// There is an exception for deleted & created object items, which we
+		// try to render as updates where that makes sense.
 		for oldI < len(old) && (lcsI >= len(lcs) || !old[oldI].RawEquals(lcs[lcsI])) {
+			// Render this as an object update if all of these are true:
+			//
+			// - the current old item is an object;
+			// - there's a current new item which is also an object;
+			// - either there are no common items left, or the current new item
+			//   doesn't equal the current common item.
+			//
+			// Why do we need the the last clause? If we have current items in all
+			// three sequences, and the current new item is equal to a common item,
+			// then we should just need to advance the old item list and we'll
+			// eventually find a common item matching both old and new.
+			//
+			// This combination of conditions allows us to render an object update
+			// diff instead of a combination of delete old & create new.
 			isObjectDiff := old[oldI].Type().IsObjectType() && newI < len(new) && new[newI].Type().IsObjectType() && (lcsI >= len(lcs) || !new[newI].RawEquals(lcs[lcsI]))
 			if isObjectDiff {
 				ret = append(ret, &plans.Change{
@@ -1832,6 +1836,8 @@ func ctySequenceDiff(old, new []cty.Value) []*plans.Change {
 				continue
 			}
 
+			// Otherwise, this item is not part of the common sequence, so
+			// render as a deletion.
 			ret = append(ret, &plans.Change{
 				Action: plans.Delete,
 				Before: old[oldI],
@@ -1847,6 +1853,9 @@ func ctySequenceDiff(old, new []cty.Value) []*plans.Change {
 			})
 			newI++
 		}
+
+		// When we've exhausted the old & new sequences of items which are not
+		// in the common subsequence, we render a common item and continue.
 		if lcsI < len(lcs) {
 			ret = append(ret, &plans.Change{
 				Action: plans.NoOp,
@@ -1991,7 +2000,7 @@ func (p *blockBodyDiffPrinter) writeSkippedAttr(skipped, indent int) {
 		}
 		p.buf.WriteString("\n")
 		p.buf.WriteString(strings.Repeat(" ", indent))
-		p.buf.WriteString(p.color.Color(fmt.Sprintf("[dark_gray]# (%d unchanged %s hidden)[reset]", skipped, noun)))
+		p.buf.WriteString(fmt.Sprintf(p.color.Color("[dark_gray]# (%d unchanged %s hidden)[reset]"), skipped, noun))
 	}
 }
 
@@ -2002,7 +2011,7 @@ func (p *blockBodyDiffPrinter) writeSkippedElems(skipped, indent int) {
 			noun = "element"
 		}
 		p.buf.WriteString(strings.Repeat(" ", indent))
-		p.buf.WriteString(p.color.Color(fmt.Sprintf("[dark_gray]# (%d unchanged %s hidden)[reset]", skipped, noun)))
+		p.buf.WriteString(fmt.Sprintf(p.color.Color("[dark_gray]# (%d unchanged %s hidden)[reset]"), skipped, noun))
 		p.buf.WriteString("\n")
 	}
 }
