@@ -331,8 +331,8 @@ func TestImport_initializationErrorShouldUnlock(t *testing.T) {
 	}
 
 	// specifically, it should fail due to a missing provider
-	msg := ui.ErrorWriter.String()
-	if want := `unknown provider "registry.terraform.io/hashicorp/unknown"`; !strings.Contains(msg, want) {
+	msg := strings.ReplaceAll(ui.ErrorWriter.String(), "\n", " ")
+	if want := `provider registry.terraform.io/hashicorp/unknown: required by this configuration but no version is selected`; !strings.Contains(msg, want) {
 		t.Errorf("incorrect message\nwant substring: %s\ngot:\n%s", want, msg)
 	}
 
