@@ -8,6 +8,7 @@ import (
 	"github.com/google/go-cmp/cmp"
 	"github.com/hashicorp/terraform/internal/addrs"
 	"github.com/hashicorp/terraform/internal/configs/configschema"
+	"github.com/hashicorp/terraform/internal/lang/marks"
 	"github.com/hashicorp/terraform/internal/states"
 	"github.com/hashicorp/terraform/internal/terraform"
 	"github.com/zclconf/go-cty/cty"
@@ -122,7 +123,7 @@ func TestMarshalAttributeValues(t *testing.T) {
 				}),
 				"baz": cty.ListVal([]cty.Value{
 					cty.StringVal("goodnight"),
-					cty.StringVal("moon").Mark("sensitive"),
+					cty.StringVal("moon").Mark(marks.Sensitive),
 				}),
 			}),
 			attributeValues{
@@ -214,7 +215,7 @@ func TestMarshalResources(t *testing.T) {
 								AttrsJSON: []byte(`{"foozles":"confuzles"}`),
 								AttrSensitivePaths: []cty.PathValueMarks{{
 									Path:  cty.Path{cty.GetAttrStep{Name: "foozles"}},
-									Marks: cty.NewValueMarks("sensitive")},
+									Marks: cty.NewValueMarks(marks.Sensitive)},
 								},
 							},
 						},
@@ -480,7 +481,7 @@ func TestMarshalResources(t *testing.T) {
 								AttrsJSON: []byte(`{"data":{"woozles":"confuzles"}}`),
 								AttrSensitivePaths: []cty.PathValueMarks{{
 									Path:  cty.Path{cty.GetAttrStep{Name: "data"}},
-									Marks: cty.NewValueMarks("sensitive")},
+									Marks: cty.NewValueMarks(marks.Sensitive)},
 								},
 							},
 						},
@@ -756,7 +757,6 @@ func testSchemas() *terraform.Schemas {
 }
 
 func TestSensitiveAsBool(t *testing.T) {
-	sensitive := "sensitive"
 	tests := []struct {
 		Input cty.Value
 		Want  cty.Value
@@ -770,16 +770,16 @@ func TestSensitiveAsBool(t *testing.T) {
 			cty.False,
 		},
 		{
-			cty.StringVal("hello").Mark(sensitive),
+			cty.StringVal("hello").Mark(marks.Sensitive),
 			cty.True,
 		},
 		{
-			cty.NullVal(cty.String).Mark(sensitive),
+			cty.NullVal(cty.String).Mark(marks.Sensitive),
 			cty.True,
 		},
 
 		{
-			cty.NullVal(cty.DynamicPseudoType).Mark(sensitive),
+			cty.NullVal(cty.DynamicPseudoType).Mark(marks.Sensitive),
 			cty.True,
 		},
 		{
@@ -787,7 +787,7 @@ func TestSensitiveAsBool(t *testing.T) {
 			cty.False,
 		},
 		{
-			cty.NullVal(cty.Object(map[string]cty.Type{"test": cty.String})).Mark(sensitive),
+			cty.NullVal(cty.Object(map[string]cty.Type{"test": cty.String})).Mark(marks.Sensitive),
 			cty.True,
 		},
 		{
@@ -795,7 +795,7 @@ func TestSensitiveAsBool(t *testing.T) {
 			cty.False,
 		},
 		{
-			cty.DynamicVal.Mark(sensitive),
+			cty.DynamicVal.Mark(marks.Sensitive),
 			cty.True,
 		},
 
@@ -804,13 +804,13 @@ func TestSensitiveAsBool(t *testing.T) {
 			cty.EmptyTupleVal,
 		},
 		{
-			cty.ListValEmpty(cty.String).Mark(sensitive),
+			cty.ListValEmpty(cty.String).Mark(marks.Sensitive),
 			cty.True,
 		},
 		{
 			cty.ListVal([]cty.Value{
 				cty.StringVal("hello"),
-				cty.StringVal("friend").Mark(sensitive),
+				cty.StringVal("friend").Mark(marks.Sensitive),
 			}),
 			cty.TupleVal([]cty.Value{
 				cty.False,
@@ -822,7 +822,7 @@ func TestSensitiveAsBool(t *testing.T) {
 			cty.EmptyTupleVal,
 		},
 		{
-			cty.SetValEmpty(cty.String).Mark(sensitive),
+			cty.SetValEmpty(cty.String).Mark(marks.Sensitive),
 			cty.True,
 		},
 		{
@@ -830,17 +830,17 @@ func TestSensitiveAsBool(t *testing.T) {
 			cty.TupleVal([]cty.Value{cty.False}),
 		},
 		{
-			cty.SetVal([]cty.Value{cty.StringVal("hello").Mark(sensitive)}),
+			cty.SetVal([]cty.Value{cty.StringVal("hello").Mark(marks.Sensitive)}),
 			cty.True,
 		},
 		{
-			cty.EmptyTupleVal.Mark(sensitive),
+			cty.EmptyTupleVal.Mark(marks.Sensitive),
 			cty.True,
 		},
 		{
 			cty.TupleVal([]cty.Value{
 				cty.StringVal("hello"),
-				cty.StringVal("friend").Mark(sensitive),
+				cty.StringVal("friend").Mark(marks.Sensitive),
 			}),
 			cty.TupleVal([]cty.Value{
 				cty.False,
@@ -852,7 +852,7 @@ func TestSensitiveAsBool(t *testing.T) {
 			cty.EmptyObjectVal,
 		},
 		{
-			cty.MapValEmpty(cty.String).Mark(sensitive),
+			cty.MapValEmpty(cty.String).Mark(marks.Sensitive),
 			cty.True,
 		},
 		{
@@ -865,7 +865,7 @@ func TestSensitiveAsBool(t *testing.T) {
 		{
 			cty.MapVal(map[string]cty.Value{
 				"greeting": cty.StringVal("hello"),
-				"animal":   cty.StringVal("horse").Mark(sensitive),
+				"animal":   cty.StringVal("horse").Mark(marks.Sensitive),
 			}),
 			cty.ObjectVal(map[string]cty.Value{
 				"animal": cty.True,
@@ -874,8 +874,8 @@ func TestSensitiveAsBool(t *testing.T) {
 		{
 			cty.MapVal(map[string]cty.Value{
 				"greeting": cty.StringVal("hello"),
-				"animal":   cty.StringVal("horse").Mark(sensitive),
-			}).Mark(sensitive),
+				"animal":   cty.StringVal("horse").Mark(marks.Sensitive),
+			}).Mark(marks.Sensitive),
 			cty.True,
 		},
 		{
@@ -892,7 +892,7 @@ func TestSensitiveAsBool(t *testing.T) {
 		{
 			cty.ObjectVal(map[string]cty.Value{
 				"greeting": cty.StringVal("hello"),
-				"animal":   cty.StringVal("horse").Mark(sensitive),
+				"animal":   cty.StringVal("horse").Mark(marks.Sensitive),
 			}),
 			cty.ObjectVal(map[string]cty.Value{
 				"animal": cty.True,
@@ -901,8 +901,8 @@ func TestSensitiveAsBool(t *testing.T) {
 		{
 			cty.ObjectVal(map[string]cty.Value{
 				"greeting": cty.StringVal("hello"),
-				"animal":   cty.StringVal("horse").Mark(sensitive),
-			}).Mark(sensitive),
+				"animal":   cty.StringVal("horse").Mark(marks.Sensitive),
+			}).Mark(marks.Sensitive),
 			cty.True,
 		},
 		{
@@ -911,7 +911,7 @@ func TestSensitiveAsBool(t *testing.T) {
 					"a": cty.UnknownVal(cty.String),
 				}),
 				cty.ObjectVal(map[string]cty.Value{
-					"a": cty.StringVal("known").Mark(sensitive),
+					"a": cty.StringVal("known").Mark(marks.Sensitive),
 				}),
 			}),
 			cty.TupleVal([]cty.Value{
@@ -925,7 +925,7 @@ func TestSensitiveAsBool(t *testing.T) {
 			cty.ListVal([]cty.Value{
 				cty.MapValEmpty(cty.String),
 				cty.MapVal(map[string]cty.Value{
-					"a": cty.StringVal("known").Mark(sensitive),
+					"a": cty.StringVal("known").Mark(marks.Sensitive),
 				}),
 				cty.MapVal(map[string]cty.Value{
 					"a": cty.UnknownVal(cty.String),
