@@ -13,6 +13,7 @@ import (
 	"github.com/hashicorp/terraform/internal/states/statefile"
 	"github.com/hashicorp/terraform/internal/terminal"
 	"github.com/hashicorp/terraform/internal/terraform"
+	"github.com/zclconf/go-cty/cty"
 )
 
 func TestOperation_stopping(t *testing.T) {
@@ -82,10 +83,8 @@ func TestOperation_planNoChanges(t *testing.T) {
 		"nothing at all in normal mode": {
 			func(schemas *terraform.Schemas) *plans.Plan {
 				return &plans.Plan{
-					UIMode:       plans.NormalMode,
-					Changes:      plans.NewChanges(),
-					PrevRunState: states.NewState(),
-					PriorState:   states.NewState(),
+					UIMode:  plans.NormalMode,
+					Changes: plans.NewChanges(),
 				}
 			},
 			"no differences, so no changes are needed.",
@@ -93,10 +92,8 @@ func TestOperation_planNoChanges(t *testing.T) {
 		"nothing at all in refresh-only mode": {
 			func(schemas *terraform.Schemas) *plans.Plan {
 				return &plans.Plan{
-					UIMode:       plans.RefreshOnlyMode,
-					Changes:      plans.NewChanges(),
-					PrevRunState: states.NewState(),
-					PriorState:   states.NewState(),
+					UIMode:  plans.RefreshOnlyMode,
+					Changes: plans.NewChanges(),
 				}
 			},
 			"Terraform has checked that the real remote objects still match",
@@ -104,152 +101,143 @@ func TestOperation_planNoChanges(t *testing.T) {
 		"nothing at all in destroy mode": {
 			func(schemas *terraform.Schemas) *plans.Plan {
 				return &plans.Plan{
-					UIMode:       plans.DestroyMode,
-					Changes:      plans.NewChanges(),
-					PrevRunState: states.NewState(),
-					PriorState:   states.NewState(),
+					UIMode:  plans.DestroyMode,
+					Changes: plans.NewChanges(),
 				}
 			},
 			"No objects need to be destroyed.",
 		},
-		"no drift to display with only deposed instances": {
-			// changes in deposed instances will cause a change in state, but
-			// have nothing to display to the user
-			func(schemas *terraform.Schemas) *plans.Plan {
-				return &plans.Plan{
-					UIMode:  plans.NormalMode,
-					Changes: plans.NewChanges(),
-					PrevRunState: states.BuildState(func(state *states.SyncState) {
-						state.SetResourceInstanceDeposed(
-							addrs.Resource{
-								Mode: addrs.ManagedResourceMode,
-								Type: "test_resource",
-								Name: "somewhere",
-							}.Instance(addrs.NoKey).Absolute(addrs.RootModuleInstance),
-							states.NewDeposedKey(),
-							&states.ResourceInstanceObjectSrc{
-								Status:    states.ObjectReady,
-								AttrsJSON: []byte(`{"foo": "ok", "bars":[]}`),
-							},
-							addrs.RootModuleInstance.ProviderConfigDefault(addrs.NewDefaultProvider("test")),
-						)
-					}),
-					PriorState: states.NewState(),
-				}
-			},
-			"no differences, so no changes are needed.",
-		},
 		"drift detected in normal mode": {
 			func(schemas *terraform.Schemas) *plans.Plan {
-				return &plans.Plan{
-					UIMode:  plans.NormalMode,
-					Changes: plans.NewChanges(),
-					PrevRunState: states.BuildState(func(state *states.SyncState) {
-						state.SetResourceInstanceCurrent(
-							addrs.Resource{
-								Mode: addrs.ManagedResourceMode,
-								Type: "test_resource",
-								Name: "somewhere",
-							}.Instance(addrs.NoKey).Absolute(addrs.RootModuleInstance),
-							&states.ResourceInstanceObjectSrc{
-								Status:    states.ObjectReady,
-								AttrsJSON: []byte(`{}`),
-							},
-							addrs.RootModuleInstance.ProviderConfigDefault(addrs.NewDefaultProvider("test")),
-						)
-					}),
-					PriorState: states.NewState(),
+				addr := addrs.Resource{
+					Mode: addrs.ManagedResourceMode,
+					Type: "test_resource",
+					Name: "somewhere",
+				}.Instance(addrs.NoKey).Absolute(addrs.RootModuleInstance)
+				schema, _ := schemas.ResourceTypeConfig(
+					addrs.NewDefaultProvider("test"),
+					addr.Resource.Resource.Mode,
+					addr.Resource.Resource.Type,
+				)
+				ty := schema.ImpliedType()
+				rc := &plans.ResourceInstanceChange{
+					Addr:        addr,
+					PrevRunAddr: addr,
+					ProviderAddr: addrs.RootModuleInstance.ProviderConfigDefault(
+						addrs.NewDefaultProvider("test"),
+					),
+					Change: plans.Change{
+						Action: plans.Update,
+						Before: cty.NullVal(ty),
+						After: cty.ObjectVal(map[string]cty.Value{
+							"id":  cty.StringVal("1234"),
+							"foo": cty.StringVal("bar"),
+						}),
+					},
 				}
-			},
-			"to update the Terraform state to match, create and apply a refresh-only plan",
-		},
-		"drift detected with deposed": {
-			func(schemas *terraform.Schemas) *plans.Plan {
+				rcs, err := rc.Encode(ty)
+				if err != nil {
+					panic(err)
+				}
+				drs := []*plans.ResourceInstanceChangeSrc{rcs}
 				return &plans.Plan{
-					UIMode:  plans.NormalMode,
-					Changes: plans.NewChanges(),
-					PrevRunState: states.BuildState(func(state *states.SyncState) {
-						state.SetResourceInstanceCurrent(
-							addrs.Resource{
-								Mode: addrs.ManagedResourceMode,
-								Type: "test_resource",
-								Name: "changes",
-							}.Instance(addrs.NoKey).Absolute(addrs.RootModuleInstance),
-							&states.ResourceInstanceObjectSrc{
-								Status:    states.ObjectReady,
-								AttrsJSON: []byte(`{"foo":"b"}`),
-							},
-							addrs.RootModuleInstance.ProviderConfigDefault(addrs.NewDefaultProvider("test")),
-						)
-						state.SetResourceInstanceDeposed(
-							addrs.Resource{
-								Mode: addrs.ManagedResourceMode,
-								Type: "test_resource",
-								Name: "broken",
-							}.Instance(addrs.NoKey).Absolute(addrs.RootModuleInstance),
-							states.NewDeposedKey(),
-							&states.ResourceInstanceObjectSrc{
-								Status:    states.ObjectReady,
-								AttrsJSON: []byte(`{"foo":"c"}`),
-							},
-							addrs.RootModuleInstance.ProviderConfigDefault(addrs.NewDefaultProvider("test")),
-						)
-					}),
-					PriorState: states.BuildState(func(state *states.SyncState) {
-						state.SetResourceInstanceCurrent(
-							addrs.Resource{
-								Mode: addrs.ManagedResourceMode,
-								Type: "test_resource",
-								Name: "changed",
-							}.Instance(addrs.NoKey).Absolute(addrs.RootModuleInstance),
-							&states.ResourceInstanceObjectSrc{
-								Status:    states.ObjectReady,
-								AttrsJSON: []byte(`{"foo":"b"}`),
-							},
-							addrs.RootModuleInstance.ProviderConfigDefault(addrs.NewDefaultProvider("test")),
-						)
-						state.SetResourceInstanceDeposed(
-							addrs.Resource{
-								Mode: addrs.ManagedResourceMode,
-								Type: "test_resource",
-								Name: "broken",
-							}.Instance(addrs.NoKey).Absolute(addrs.RootModuleInstance),
-							states.NewDeposedKey(),
-							&states.ResourceInstanceObjectSrc{
-								Status:    states.ObjectReady,
-								AttrsJSON: []byte(`{"foo":"d"}`),
-							},
-							addrs.RootModuleInstance.ProviderConfigDefault(addrs.NewDefaultProvider("test")),
-						)
-					}),
+					UIMode:           plans.NormalMode,
+					Changes:          plans.NewChanges(),
+					DriftedResources: drs,
 				}
 			},
 			"to update the Terraform state to match, create and apply a refresh-only plan",
 		},
 		"drift detected in refresh-only mode": {
 			func(schemas *terraform.Schemas) *plans.Plan {
+				addr := addrs.Resource{
+					Mode: addrs.ManagedResourceMode,
+					Type: "test_resource",
+					Name: "somewhere",
+				}.Instance(addrs.NoKey).Absolute(addrs.RootModuleInstance)
+				schema, _ := schemas.ResourceTypeConfig(
+					addrs.NewDefaultProvider("test"),
+					addr.Resource.Resource.Mode,
+					addr.Resource.Resource.Type,
+				)
+				ty := schema.ImpliedType()
+				rc := &plans.ResourceInstanceChange{
+					Addr:        addr,
+					PrevRunAddr: addr,
+					ProviderAddr: addrs.RootModuleInstance.ProviderConfigDefault(
+						addrs.NewDefaultProvider("test"),
+					),
+					Change: plans.Change{
+						Action: plans.Update,
+						Before: cty.NullVal(ty),
+						After: cty.ObjectVal(map[string]cty.Value{
+							"id":  cty.StringVal("1234"),
+							"foo": cty.StringVal("bar"),
+						}),
+					},
+				}
+				rcs, err := rc.Encode(ty)
+				if err != nil {
+					panic(err)
+				}
+				drs := []*plans.ResourceInstanceChangeSrc{rcs}
 				return &plans.Plan{
-					UIMode:  plans.RefreshOnlyMode,
-					Changes: plans.NewChanges(),
-					PrevRunState: states.BuildState(func(state *states.SyncState) {
-						state.SetResourceInstanceCurrent(
-							addrs.Resource{
-								Mode: addrs.ManagedResourceMode,
-								Type: "test_resource",
-								Name: "somewhere",
-							}.Instance(addrs.NoKey).Absolute(addrs.RootModuleInstance),
-							&states.ResourceInstanceObjectSrc{
-								Status:    states.ObjectReady,
-								AttrsJSON: []byte(`{}`),
-							},
-							addrs.RootModuleInstance.ProviderConfigDefault(addrs.NewDefaultProvider("test")),
-						)
-					}),
-					PriorState: states.NewState(),
+					UIMode:           plans.RefreshOnlyMode,
+					Changes:          plans.NewChanges(),
+					DriftedResources: drs,
 				}
 			},
 			"If you were expecting these changes then you can apply this plan",
 		},
+		"move-only changes in refresh-only mode": {
+			func(schemas *terraform.Schemas) *plans.Plan {
+				addr := addrs.Resource{
+					Mode: addrs.ManagedResourceMode,
+					Type: "test_resource",
+					Name: "somewhere",
+				}.Instance(addrs.NoKey).Absolute(addrs.RootModuleInstance)
+				addrPrev := addrs.Resource{
+					Mode: addrs.ManagedResourceMode,
+					Type: "test_resource",
+					Name: "anywhere",
+				}.Instance(addrs.NoKey).Absolute(addrs.RootModuleInstance)
+				schema, _ := schemas.ResourceTypeConfig(
+					addrs.NewDefaultProvider("test"),
+					addr.Resource.Resource.Mode,
+					addr.Resource.Resource.Type,
+				)
+				ty := schema.ImpliedType()
+				rc := &plans.ResourceInstanceChange{
+					Addr:        addr,
+					PrevRunAddr: addrPrev,
+					ProviderAddr: addrs.RootModuleInstance.ProviderConfigDefault(
+						addrs.NewDefaultProvider("test"),
+					),
+					Change: plans.Change{
+						Action: plans.NoOp,
+						Before: cty.ObjectVal(map[string]cty.Value{
+							"id":  cty.StringVal("1234"),
+							"foo": cty.StringVal("bar"),
+						}),
+						After: cty.ObjectVal(map[string]cty.Value{
+							"id":  cty.StringVal("1234"),
+							"foo": cty.StringVal("bar"),
+						}),
+					},
+				}
+				rcs, err := rc.Encode(ty)
+				if err != nil {
+					panic(err)
+				}
+				drs := []*plans.ResourceInstanceChangeSrc{rcs}
+				return &plans.Plan{
+					UIMode:           plans.RefreshOnlyMode,
+					Changes:          plans.NewChanges(),
+					DriftedResources: drs,
+				}
+			},
+			"test_resource.anywhere has moved to test_resource.somewhere",
+		},
 		"drift detected in destroy mode": {
 			func(schemas *terraform.Schemas) *plans.Plan {
 				return &plans.Plan{
@@ -491,29 +479,35 @@ func TestOperationJSON_plan(t *testing.T) {
 		Changes: &plans.Changes{
 			Resources: []*plans.ResourceInstanceChangeSrc{
 				{
-					Addr:      boop.Instance(addrs.IntKey(0)).Absolute(root),
-					ChangeSrc: plans.ChangeSrc{Action: plans.CreateThenDelete},
+					Addr:        boop.Instance(addrs.IntKey(0)).Absolute(root),
+					PrevRunAddr: boop.Instance(addrs.IntKey(0)).Absolute(root),
+					ChangeSrc:   plans.ChangeSrc{Action: plans.CreateThenDelete},
 				},
 				{
-					Addr:      boop.Instance(addrs.IntKey(1)).Absolute(root),
-					ChangeSrc: plans.ChangeSrc{Action: plans.Create},
+					Addr:        boop.Instance(addrs.IntKey(1)).Absolute(root),
+					PrevRunAddr: boop.Instance(addrs.IntKey(1)).Absolute(root),
+					ChangeSrc:   plans.ChangeSrc{Action: plans.Create},
 				},
 				{
-					Addr:      boop.Instance(addrs.IntKey(0)).Absolute(vpc),
-					ChangeSrc: plans.ChangeSrc{Action: plans.Delete},
+					Addr:        boop.Instance(addrs.IntKey(0)).Absolute(vpc),
+					PrevRunAddr: boop.Instance(addrs.IntKey(0)).Absolute(vpc),
+					ChangeSrc:   plans.ChangeSrc{Action: plans.Delete},
 				},
 				{
-					Addr:      beep.Instance(addrs.NoKey).Absolute(root),
-					ChangeSrc: plans.ChangeSrc{Action: plans.DeleteThenCreate},
+					Addr:        beep.Instance(addrs.NoKey).Absolute(root),
+					PrevRunAddr: beep.Instance(addrs.NoKey).Absolute(root),
+					ChangeSrc:   plans.ChangeSrc{Action: plans.DeleteThenCreate},
 				},
 				{
-					Addr:      beep.Instance(addrs.NoKey).Absolute(vpc),
-					ChangeSrc: plans.ChangeSrc{Action: plans.Update},
+					Addr:        beep.Instance(addrs.NoKey).Absolute(vpc),
+					PrevRunAddr: beep.Instance(addrs.NoKey).Absolute(vpc),
+					ChangeSrc:   plans.ChangeSrc{Action: plans.Update},
 				},
 				// Data source deletion should not show up in the logs
 				{
-					Addr:      derp.Instance(addrs.NoKey).Absolute(root),
-					ChangeSrc: plans.ChangeSrc{Action: plans.Delete},
+					Addr:        derp.Instance(addrs.NoKey).Absolute(root),
+					PrevRunAddr: derp.Instance(addrs.NoKey).Absolute(root),
+					ChangeSrc:   plans.ChangeSrc{Action: plans.Delete},
 				},
 			},
 		},
@@ -635,74 +629,175 @@ func TestOperationJSON_plan(t *testing.T) {
 	testJSONViewOutputEquals(t, done(t).Stdout(), want)
 }
 
-func TestOperationJSON_planDrift(t *testing.T) {
+func TestOperationJSON_planDriftWithMove(t *testing.T) {
 	streams, done := terminal.StreamsForTesting(t)
 	v := &OperationJSON{view: NewJSONView(NewView(streams))}
 
 	root := addrs.RootModuleInstance
 	boop := addrs.Resource{Mode: addrs.ManagedResourceMode, Type: "test_resource", Name: "boop"}
 	beep := addrs.Resource{Mode: addrs.ManagedResourceMode, Type: "test_resource", Name: "beep"}
-	derp := addrs.Resource{Mode: addrs.ManagedResourceMode, Type: "test_resource", Name: "derp"}
+	blep := addrs.Resource{Mode: addrs.ManagedResourceMode, Type: "test_resource", Name: "blep"}
+	honk := addrs.Resource{Mode: addrs.ManagedResourceMode, Type: "test_resource", Name: "honk"}
 
 	plan := &plans.Plan{
+		UIMode: plans.NormalMode,
+		Changes: &plans.Changes{
+			Resources: []*plans.ResourceInstanceChangeSrc{
+				{
+					Addr:        honk.Instance(addrs.StringKey("bonk")).Absolute(root),
+					PrevRunAddr: honk.Instance(addrs.IntKey(0)).Absolute(root),
+					ChangeSrc:   plans.ChangeSrc{Action: plans.NoOp},
+				},
+			},
+		},
+		DriftedResources: []*plans.ResourceInstanceChangeSrc{
+			{
+				Addr:        beep.Instance(addrs.NoKey).Absolute(root),
+				PrevRunAddr: beep.Instance(addrs.NoKey).Absolute(root),
+				ChangeSrc:   plans.ChangeSrc{Action: plans.Delete},
+			},
+			{
+				Addr:        boop.Instance(addrs.NoKey).Absolute(root),
+				PrevRunAddr: blep.Instance(addrs.NoKey).Absolute(root),
+				ChangeSrc:   plans.ChangeSrc{Action: plans.Update},
+			},
+			// Move-only resource drift should not be present in normal mode plans
+			{
+				Addr:        honk.Instance(addrs.StringKey("bonk")).Absolute(root),
+				PrevRunAddr: honk.Instance(addrs.IntKey(0)).Absolute(root),
+				ChangeSrc:   plans.ChangeSrc{Action: plans.NoOp},
+			},
+		},
+	}
+	v.Plan(plan, testSchemas())
+
+	want := []map[string]interface{}{
+		// Drift detected: delete
+		{
+			"@level":   "info",
+			"@message": "test_resource.beep: Drift detected (delete)",
+			"@module":  "terraform.ui",
+			"type":     "resource_drift",
+			"change": map[string]interface{}{
+				"action": "delete",
+				"resource": map[string]interface{}{
+					"addr":             "test_resource.beep",
+					"implied_provider": "test",
+					"module":           "",
+					"resource":         "test_resource.beep",
+					"resource_key":     nil,
+					"resource_name":    "beep",
+					"resource_type":    "test_resource",
+				},
+			},
+		},
+		// Drift detected: update with move
+		{
+			"@level":   "info",
+			"@message": "test_resource.boop: Drift detected (update)",
+			"@module":  "terraform.ui",
+			"type":     "resource_drift",
+			"change": map[string]interface{}{
+				"action": "update",
+				"resource": map[string]interface{}{
+					"addr":             "test_resource.boop",
+					"implied_provider": "test",
+					"module":           "",
+					"resource":         "test_resource.boop",
+					"resource_key":     nil,
+					"resource_name":    "boop",
+					"resource_type":    "test_resource",
+				},
+				"previous_resource": map[string]interface{}{
+					"addr":             "test_resource.blep",
+					"implied_provider": "test",
+					"module":           "",
+					"resource":         "test_resource.blep",
+					"resource_key":     nil,
+					"resource_name":    "blep",
+					"resource_type":    "test_resource",
+				},
+			},
+		},
+		// Move-only change
+		{
+			"@level":   "info",
+			"@message": `test_resource.honk["bonk"]: Plan to move`,
+			"@module":  "terraform.ui",
+			"type":     "planned_change",
+			"change": map[string]interface{}{
+				"action": "move",
+				"resource": map[string]interface{}{
+					"addr":             `test_resource.honk["bonk"]`,
+					"implied_provider": "test",
+					"module":           "",
+					"resource":         `test_resource.honk["bonk"]`,
+					"resource_key":     "bonk",
+					"resource_name":    "honk",
+					"resource_type":    "test_resource",
+				},
+				"previous_resource": map[string]interface{}{
+					"addr":             `test_resource.honk[0]`,
+					"implied_provider": "test",
+					"module":           "",
+					"resource":         `test_resource.honk[0]`,
+					"resource_key":     float64(0),
+					"resource_name":    "honk",
+					"resource_type":    "test_resource",
+				},
+			},
+		},
+		// No changes
+		{
+			"@level":   "info",
+			"@message": "Plan: 0 to add, 0 to change, 0 to destroy.",
+			"@module":  "terraform.ui",
+			"type":     "change_summary",
+			"changes": map[string]interface{}{
+				"operation": "plan",
+				"add":       float64(0),
+				"change":    float64(0),
+				"remove":    float64(0),
+			},
+		},
+	}
+
+	testJSONViewOutputEquals(t, done(t).Stdout(), want)
+}
+
+func TestOperationJSON_planDriftWithMoveRefreshOnly(t *testing.T) {
+	streams, done := terminal.StreamsForTesting(t)
+	v := &OperationJSON{view: NewJSONView(NewView(streams))}
+
+	root := addrs.RootModuleInstance
+	boop := addrs.Resource{Mode: addrs.ManagedResourceMode, Type: "test_resource", Name: "boop"}
+	beep := addrs.Resource{Mode: addrs.ManagedResourceMode, Type: "test_resource", Name: "beep"}
+	blep := addrs.Resource{Mode: addrs.ManagedResourceMode, Type: "test_resource", Name: "blep"}
+	honk := addrs.Resource{Mode: addrs.ManagedResourceMode, Type: "test_resource", Name: "honk"}
+
+	plan := &plans.Plan{
+		UIMode: plans.RefreshOnlyMode,
 		Changes: &plans.Changes{
 			Resources: []*plans.ResourceInstanceChangeSrc{},
 		},
-		PrevRunState: states.BuildState(func(state *states.SyncState) {
-			// Update
-			state.SetResourceInstanceCurrent(
-				boop.Instance(addrs.NoKey).Absolute(root),
-				&states.ResourceInstanceObjectSrc{
-					Status:    states.ObjectReady,
-					AttrsJSON: []byte(`{"foo":"bar"}`),
-				},
-				root.ProviderConfigDefault(addrs.NewDefaultProvider("test")),
-			)
-			// Delete
-			state.SetResourceInstanceCurrent(
-				beep.Instance(addrs.NoKey).Absolute(root),
-				&states.ResourceInstanceObjectSrc{
-					Status:    states.ObjectReady,
-					AttrsJSON: []byte(`{"foo":"boop"}`),
-				},
-				root.ProviderConfigDefault(addrs.NewDefaultProvider("test")),
-			)
-			// No-op
-			state.SetResourceInstanceCurrent(
-				derp.Instance(addrs.NoKey).Absolute(root),
-				&states.ResourceInstanceObjectSrc{
-					Status:    states.ObjectReady,
-					AttrsJSON: []byte(`{"foo":"boop"}`),
-				},
-				root.ProviderConfigDefault(addrs.NewDefaultProvider("test")),
-			)
-		}),
-		PriorState: states.BuildState(func(state *states.SyncState) {
-			// Update
-			state.SetResourceInstanceCurrent(
-				boop.Instance(addrs.NoKey).Absolute(root),
-				&states.ResourceInstanceObjectSrc{
-					Status:    states.ObjectReady,
-					AttrsJSON: []byte(`{"foo":"baz"}`),
-				},
-				root.ProviderConfigDefault(addrs.NewDefaultProvider("test")),
-			)
-			// Delete
-			state.SetResourceInstanceCurrent(
-				beep.Instance(addrs.NoKey).Absolute(root),
-				nil,
-				root.ProviderConfigDefault(addrs.NewDefaultProvider("test")),
-			)
-			// No-op
-			state.SetResourceInstanceCurrent(
-				derp.Instance(addrs.NoKey).Absolute(root),
-				&states.ResourceInstanceObjectSrc{
-					Status:    states.ObjectReady,
-					AttrsJSON: []byte(`{"foo":"boop"}`),
-				},
-				root.ProviderConfigDefault(addrs.NewDefaultProvider("test")),
-			)
-		}),
+		DriftedResources: []*plans.ResourceInstanceChangeSrc{
+			{
+				Addr:        beep.Instance(addrs.NoKey).Absolute(root),
+				PrevRunAddr: beep.Instance(addrs.NoKey).Absolute(root),
+				ChangeSrc:   plans.ChangeSrc{Action: plans.Delete},
+			},
+			{
+				Addr:        boop.Instance(addrs.NoKey).Absolute(root),
+				PrevRunAddr: blep.Instance(addrs.NoKey).Absolute(root),
+				ChangeSrc:   plans.ChangeSrc{Action: plans.Update},
+			},
+			// Move-only resource drift should be present in refresh-only plans
+			{
+				Addr:        honk.Instance(addrs.StringKey("bonk")).Absolute(root),
+				PrevRunAddr: honk.Instance(addrs.IntKey(0)).Absolute(root),
+				ChangeSrc:   plans.ChangeSrc{Action: plans.NoOp},
+			},
+		},
 	}
 	v.Plan(plan, testSchemas())
 
@@ -743,6 +838,43 @@ func TestOperationJSON_planDrift(t *testing.T) {
 					"resource_name":    "boop",
 					"resource_type":    "test_resource",
 				},
+				"previous_resource": map[string]interface{}{
+					"addr":             "test_resource.blep",
+					"implied_provider": "test",
+					"module":           "",
+					"resource":         "test_resource.blep",
+					"resource_key":     nil,
+					"resource_name":    "blep",
+					"resource_type":    "test_resource",
+				},
+			},
+		},
+		// Drift detected: Move-only change
+		{
+			"@level":   "info",
+			"@message": `test_resource.honk["bonk"]: Drift detected (move)`,
+			"@module":  "terraform.ui",
+			"type":     "resource_drift",
+			"change": map[string]interface{}{
+				"action": "move",
+				"resource": map[string]interface{}{
+					"addr":             `test_resource.honk["bonk"]`,
+					"implied_provider": "test",
+					"module":           "",
+					"resource":         `test_resource.honk["bonk"]`,
+					"resource_key":     "bonk",
+					"resource_name":    "honk",
+					"resource_type":    "test_resource",
+				},
+				"previous_resource": map[string]interface{}{
+					"addr":             `test_resource.honk[0]`,
+					"implied_provider": "test",
+					"module":           "",
+					"resource":         `test_resource.honk[0]`,
+					"resource_key":     float64(0),
+					"resource_name":    "honk",
+					"resource_type":    "test_resource",
+				},
 			},
 		},
 		// No changes
@@ -763,6 +895,90 @@ func TestOperationJSON_planDrift(t *testing.T) {
 	testJSONViewOutputEquals(t, done(t).Stdout(), want)
 }
 
+func TestOperationJSON_planOutputChanges(t *testing.T) {
+	streams, done := terminal.StreamsForTesting(t)
+	v := &OperationJSON{view: NewJSONView(NewView(streams))}
+
+	root := addrs.RootModuleInstance
+
+	plan := &plans.Plan{
+		Changes: &plans.Changes{
+			Resources: []*plans.ResourceInstanceChangeSrc{},
+			Outputs: []*plans.OutputChangeSrc{
+				{
+					Addr: root.OutputValue("boop"),
+					ChangeSrc: plans.ChangeSrc{
+						Action: plans.NoOp,
+					},
+				},
+				{
+					Addr: root.OutputValue("beep"),
+					ChangeSrc: plans.ChangeSrc{
+						Action: plans.Create,
+					},
+				},
+				{
+					Addr: root.OutputValue("bonk"),
+					ChangeSrc: plans.ChangeSrc{
+						Action: plans.Delete,
+					},
+				},
+				{
+					Addr: root.OutputValue("honk"),
+					ChangeSrc: plans.ChangeSrc{
+						Action: plans.Update,
+					},
+					Sensitive: true,
+				},
+			},
+		},
+	}
+	v.Plan(plan, testSchemas())
+
+	want := []map[string]interface{}{
+		// No resource changes
+		{
+			"@level":   "info",
+			"@message": "Plan: 0 to add, 0 to change, 0 to destroy.",
+			"@module":  "terraform.ui",
+			"type":     "change_summary",
+			"changes": map[string]interface{}{
+				"operation": "plan",
+				"add":       float64(0),
+				"change":    float64(0),
+				"remove":    float64(0),
+			},
+		},
+		// Output changes
+		{
+			"@level":   "info",
+			"@message": "Outputs: 4",
+			"@module":  "terraform.ui",
+			"type":     "outputs",
+			"outputs": map[string]interface{}{
+				"boop": map[string]interface{}{
+					"action":    "noop",
+					"sensitive": false,
+				},
+				"beep": map[string]interface{}{
+					"action":    "create",
+					"sensitive": false,
+				},
+				"bonk": map[string]interface{}{
+					"action":    "delete",
+					"sensitive": false,
+				},
+				"honk": map[string]interface{}{
+					"action":    "update",
+					"sensitive": true,
+				},
+			},
+		},
+	}
+
+	testJSONViewOutputEquals(t, done(t).Stdout(), want)
+}
+
 func TestOperationJSON_plannedChange(t *testing.T) {
 	streams, done := terminal.StreamsForTesting(t)
 	v := &OperationJSON{view: NewJSONView(NewView(streams))}
@@ -774,20 +990,23 @@ func TestOperationJSON_plannedChange(t *testing.T) {
 	// Replace requested by user
 	v.PlannedChange(&plans.ResourceInstanceChangeSrc{
 		Addr:         boop.Instance(addrs.IntKey(0)).Absolute(root),
+		PrevRunAddr:  boop.Instance(addrs.IntKey(0)).Absolute(root),
 		ChangeSrc:    plans.ChangeSrc{Action: plans.DeleteThenCreate},
 		ActionReason: plans.ResourceInstanceReplaceByRequest,
 	})
 
 	// Simple create
 	v.PlannedChange(&plans.ResourceInstanceChangeSrc{
-		Addr:      boop.Instance(addrs.IntKey(1)).Absolute(root),
-		ChangeSrc: plans.ChangeSrc{Action: plans.Create},
+		Addr:        boop.Instance(addrs.IntKey(1)).Absolute(root),
+		PrevRunAddr: boop.Instance(addrs.IntKey(1)).Absolute(root),
+		ChangeSrc:   plans.ChangeSrc{Action: plans.Create},
 	})
 
 	// Data source deletion
 	v.PlannedChange(&plans.ResourceInstanceChangeSrc{
-		Addr:      derp.Instance(addrs.NoKey).Absolute(root),
-		ChangeSrc: plans.ChangeSrc{Action: plans.Delete},
+		Addr:        derp.Instance(addrs.NoKey).Absolute(root),
+		PrevRunAddr: derp.Instance(addrs.NoKey).Absolute(root),
+		ChangeSrc:   plans.ChangeSrc{Action: plans.Delete},
 	})
 
 	// Expect only two messages, as the data source deletion should be a no-op
