@@ -5,10 +5,12 @@ import (
 	"strings"
 	"testing"
 
+	"github.com/google/go-cmp/cmp"
+	"github.com/zclconf/go-cty/cty"
+
 	"github.com/hashicorp/terraform/internal/addrs"
 	"github.com/hashicorp/terraform/internal/plans"
 	"github.com/hashicorp/terraform/internal/states"
-	"github.com/zclconf/go-cty/cty"
 )
 
 func TestApplyGraphBuilder_impl(t *testing.T) {
@@ -46,10 +48,9 @@ func TestApplyGraphBuilder(t *testing.T) {
 	}
 
 	b := &ApplyGraphBuilder{
-		Config:     testModule(t, "graph-builder-apply-basic"),
-		Changes:    changes,
-		Components: simpleMockComponentFactory(),
-		Schemas:    simpleTestSchemas(),
+		Config:  testModule(t, "graph-builder-apply-basic"),
+		Changes: changes,
+		Plugins: simpleMockPluginLibrary(),
 	}
 
 	g, err := b.Build(addrs.RootModuleInstance)
@@ -61,11 +62,10 @@ func TestApplyGraphBuilder(t *testing.T) {
 		t.Fatalf("wrong path %q", g.Path.String())
 	}
 
-	actual := strings.TrimSpace(g.String())
-
-	expected := strings.TrimSpace(testApplyGraphBuilderStr)
-	if actual != expected {
-		t.Fatalf("wrong result\n\ngot:\n%s\n\nwant:\n%s", actual, expected)
+	got := strings.TrimSpace(g.String())
+	want := strings.TrimSpace(testApplyGraphBuilderStr)
+	if diff := cmp.Diff(want, got); diff != "" {
+		t.Fatalf("wrong result\n%s", diff)
 	}
 }
 
@@ -110,11 +110,10 @@ func TestApplyGraphBuilder_depCbd(t *testing.T) {
 	)
 
 	b := &ApplyGraphBuilder{
-		Config:     testModule(t, "graph-builder-apply-dep-cbd"),
-		Changes:    changes,
-		Components: simpleMockComponentFactory(),
-		Schemas:    simpleTestSchemas(),
-		State:      state,
+		Config:  testModule(t, "graph-builder-apply-dep-cbd"),
+		Changes: changes,
+		Plugins: simpleMockPluginLibrary(),
+		State:   state,
 	}
 
 	g, err := b.Build(addrs.RootModuleInstance)
@@ -184,10 +183,9 @@ func TestApplyGraphBuilder_doubleCBD(t *testing.T) {
 	}
 
 	b := &ApplyGraphBuilder{
-		Config:     testModule(t, "graph-builder-apply-double-cbd"),
-		Changes:    changes,
-		Components: simpleMockComponentFactory(),
-		Schemas:    simpleTestSchemas(),
+		Config:  testModule(t, "graph-builder-apply-double-cbd"),
+		Changes: changes,
+		Plugins: simpleMockPluginLibrary(),
 	}
 
 	g, err := b.Build(addrs.RootModuleInstance)
@@ -278,11 +276,10 @@ func TestApplyGraphBuilder_destroyStateOnly(t *testing.T) {
 	)
 
 	b := &ApplyGraphBuilder{
-		Config:     testModule(t, "empty"),
-		Changes:    changes,
-		State:      state,
-		Components: simpleMockComponentFactory(),
-		Schemas:    simpleTestSchemas(),
+		Config:  testModule(t, "empty"),
+		Changes: changes,
+		State:   state,
+		Plugins: simpleMockPluginLibrary(),
 	}
 
 	g, diags := b.Build(addrs.RootModuleInstance)
@@ -341,11 +338,10 @@ func TestApplyGraphBuilder_destroyCount(t *testing.T) {
 	)
 
 	b := &ApplyGraphBuilder{
-		Config:     testModule(t, "graph-builder-apply-count"),
-		Changes:    changes,
-		Components: simpleMockComponentFactory(),
-		Schemas:    simpleTestSchemas(),
-		State:      state,
+		Config:  testModule(t, "graph-builder-apply-count"),
+		Changes: changes,
+		Plugins: simpleMockPluginLibrary(),
+		State:   state,
 	}
 
 	g, err := b.Build(addrs.RootModuleInstance)
@@ -357,10 +353,10 @@ func TestApplyGraphBuilder_destroyCount(t *testing.T) {
 		t.Fatalf("wrong module path %q", g.Path)
 	}
 
-	actual := strings.TrimSpace(g.String())
-	expected := strings.TrimSpace(testApplyGraphBuilderDestroyCountStr)
-	if actual != expected {
-		t.Fatalf("wrong result\n\ngot:\n%s\n\nwant:\n%s", actual, expected)
+	got := strings.TrimSpace(g.String())
+	want := strings.TrimSpace(testApplyGraphBuilderDestroyCountStr)
+	if diff := cmp.Diff(want, got); diff != "" {
+		t.Fatalf("wrong result\n%s", diff)
 	}
 }
 
@@ -404,11 +400,10 @@ func TestApplyGraphBuilder_moduleDestroy(t *testing.T) {
 	)
 
 	b := &ApplyGraphBuilder{
-		Config:     testModule(t, "graph-builder-apply-module-destroy"),
-		Changes:    changes,
-		Components: simpleMockComponentFactory(),
-		Schemas:    simpleTestSchemas(),
-		State:      state,
+		Config:  testModule(t, "graph-builder-apply-module-destroy"),
+		Changes: changes,
+		Plugins: simpleMockPluginLibrary(),
+		State:   state,
 	}
 
 	g, err := b.Build(addrs.RootModuleInstance)
@@ -442,10 +437,9 @@ func TestApplyGraphBuilder_targetModule(t *testing.T) {
 	}
 
 	b := &ApplyGraphBuilder{
-		Config:     testModule(t, "graph-builder-apply-target-module"),
-		Changes:    changes,
-		Components: simpleMockComponentFactory(),
-		Schemas:    simpleTestSchemas(),
+		Config:  testModule(t, "graph-builder-apply-target-module"),
+		Changes: changes,
+		Plugins: simpleMockPluginLibrary(),
 		Targets: []addrs.Targetable{
 			addrs.RootModuleInstance.Child("child2", addrs.NoKey),
 		},
@@ -522,7 +516,7 @@ func TestApplyGraphBuilder_updateFromOrphan(t *testing.T) {
 			Status:    states.ObjectReady,
 			AttrsJSON: []byte(`{"id":"b_id","test_string":"a_id"}`),
 			Dependencies: []addrs.ConfigResource{
-				addrs.ConfigResource{
+				{
 					Resource: addrs.Resource{
 						Mode: addrs.ManagedResourceMode,
 						Type: "test_object",
@@ -539,11 +533,10 @@ func TestApplyGraphBuilder_updateFromOrphan(t *testing.T) {
 	)
 
 	b := &ApplyGraphBuilder{
-		Config:     testModule(t, "graph-builder-apply-orphan-update"),
-		Changes:    changes,
-		Components: simpleMockComponentFactory(),
-		Schemas:    schemas,
-		State:      state,
+		Config:  testModule(t, "graph-builder-apply-orphan-update"),
+		Changes: changes,
+		Plugins: simpleMockPluginLibrary(),
+		State:   state,
 	}
 
 	g, err := b.Build(addrs.RootModuleInstance)
@@ -626,7 +619,7 @@ func TestApplyGraphBuilder_updateFromCBDOrphan(t *testing.T) {
 			Status:    states.ObjectReady,
 			AttrsJSON: []byte(`{"id":"b_id","test_string":"a_id"}`),
 			Dependencies: []addrs.ConfigResource{
-				addrs.ConfigResource{
+				{
 					Resource: addrs.Resource{
 						Mode: addrs.ManagedResourceMode,
 						Type: "test_object",
@@ -640,11 +633,10 @@ func TestApplyGraphBuilder_updateFromCBDOrphan(t *testing.T) {
 	)
 
 	b := &ApplyGraphBuilder{
-		Config:     testModule(t, "graph-builder-apply-orphan-update"),
-		Changes:    changes,
-		Components: simpleMockComponentFactory(),
-		Schemas:    schemas,
-		State:      state,
+		Config:  testModule(t, "graph-builder-apply-orphan-update"),
+		Changes: changes,
+		Plugins: simpleMockPluginLibrary(),
+		State:   state,
 	}
 
 	g, err := b.Build(addrs.RootModuleInstance)
@@ -691,11 +683,10 @@ func TestApplyGraphBuilder_orphanedWithProvider(t *testing.T) {
 	)
 
 	b := &ApplyGraphBuilder{
-		Config:     testModule(t, "graph-builder-orphan-alias"),
-		Changes:    changes,
-		Components: simpleMockComponentFactory(),
-		Schemas:    simpleTestSchemas(),
-		State:      state,
+		Config:  testModule(t, "graph-builder-orphan-alias"),
+		Changes: changes,
+		Plugins: simpleMockPluginLibrary(),
+		State:   state,
 	}
 
 	g, err := b.Build(addrs.RootModuleInstance)
@@ -709,9 +700,6 @@ func TestApplyGraphBuilder_orphanedWithProvider(t *testing.T) {
 }
 
 const testApplyGraphBuilderStr = `
-meta.count-boundary (EachMode fixup)
-  module.child (close)
-  test_object.other
 module.child (close)
   module.child.test_object.other
 module.child (expand)
@@ -731,7 +719,7 @@ provider["registry.terraform.io/hashicorp/test"] (close)
   module.child.test_object.other
   test_object.other
 root
-  meta.count-boundary (EachMode fixup)
+  module.child (close)
   provider["registry.terraform.io/hashicorp/test"] (close)
 test_object.create
   test_object.create (expand)
@@ -745,13 +733,10 @@ test_object.other (expand)
 `
 
 const testApplyGraphBuilderDestroyCountStr = `
-meta.count-boundary (EachMode fixup)
-  test_object.B
 provider["registry.terraform.io/hashicorp/test"]
 provider["registry.terraform.io/hashicorp/test"] (close)
   test_object.B
 root
-  meta.count-boundary (EachMode fixup)
   provider["registry.terraform.io/hashicorp/test"] (close)
 test_object.A (expand)
   provider["registry.terraform.io/hashicorp/test"]
