@@ -6,20 +6,6 @@ import (
 	"github.com/hashicorp/terraform/internal/dag"
 )
 
-// testGraphContains is an assertion helper that tests that a node is
-// contained in the graph.
-func testGraphContains(t *testing.T, g *Graph, name string) {
-	for _, v := range g.Vertices() {
-		if dag.VertexName(v) == name {
-			return
-		}
-	}
-
-	t.Fatalf(
-		"Expected %q in:\n\n%s",
-		name, g.String())
-}
-
 // testGraphnotContains is an assertion helper that tests that a node is
 // NOT contained in the graph.
 func testGraphNotContains(t *testing.T, g *Graph, name string) {
