@@ -19,9 +19,7 @@ func TestRootTransformer(t *testing.T) {
 	}
 
 	{
-		transform := &MissingProviderTransformer{
-			Providers: []string{"aws", "do"},
-		}
+		transform := &MissingProviderTransformer{}
 		if err := transform.Transform(&g); err != nil {
 			t.Fatalf("err: %s", err)
 		}
